//Kevin Rye, 2015
//compiled on Arduino 1.6.5
//V1.0

#include <font4x6.h>
#include <font8x8.h>
#include <font16x32.h>
#include <retro8x16.h>
#include <Wire.h>
#include <Sodaq_DS3231.h>
#include <TVout.h>

TVout TV;

//RTC data
int seconds = 0;
int minutes = 0;
int hours = 12;
int adjHours = 0;
int dayOfWeek = 1;
int dayOfMonth = 1;
int month = 1;
int year = 15; //not actually displayed

int hourButton = 2;
int minuteButton = 3;
int dayButton = 4;
int monthButton = 5;
int dateButton = 6;
int modeButton = 8;

//temp
int tempC;
int tempF;

//timer
long previousMillis = 0;
long interval = 900;

void setup() {

  pinMode(hourButton, INPUT);
  pinMode(minuteButton, INPUT);
  pinMode(dayButton, INPUT);
  pinMode(monthButton, INPUT);
  pinMode(dateButton, INPUT);
  pinMode(modeButton, INPUT);

  TV.begin(NTSC, 120, 96);
  Wire.begin();
  rtc.begin();

} //end setup

void loop () {

  //get data from DS3231
  get_time();
  get_date();

  TV.delay_frame(10); //stops screen flicker

  tempC = rtc.getTemperature();
  tempF = (tempC * 1.8) + 32.0; // Convert Celcius to Fahrenheit

  //pass RTC values into a new variable -
  //this way one variable stores time in 24 hour mode and another that can be converted to 12 hour mode
  //time is monitored 'in the background' in 24 hour mode using the RTC, but is displayed in an adjusted 12-hr format
  adjHours = hours;

  //convert to 12 hr mode for display
  if (hours < 1) {
    adjHours = 12;
  }

  if ((hours > 12) && (hours < 24)) {
    adjHours = hours - 12;
  }

  //FIRST LINE  //display RTC data on screen
  TV.select_font(retro8x16);

  //time
  //put in code to shift time over a hair when only displaying 1 digit
  if (adjHours < 10) {
    TV.print(26, 5, adjHours);
  }
  else {
    TV.print(18, 5, adjHours);
  }

  TV.print(":");

  if (minutes < 10) {
    TV.print("0");
    TV.print(minutes);
  }
  else {
    TV.print(minutes);
  }

  TV.print(":");

  if (seconds < 10) {
    TV.print("0");
    TV.print(seconds);
  }
  else {
    TV.print(seconds);
  }

  //AM/PM
  TV.select_font(font4x6);
  if (hours > 11) {

    TV.print(" PM");
  }
  else {
    TV.print(" AM");
  }

  //SECOND LINE
  TV.draw_line(15, 25, 96, 25, 1); //draw a white line 1 is white, 0 is black, 1 is white

  TV.select_font(font4x6);
  TV.print(20, 30, "DAY");
  TV.print(48, 30, "MONTH");
  TV.print(78, 30, "DATE");

  //THIRD LINE
  //day of week
  TV.select_font(retro8x16);

  switch (dayOfWeek) {
    case 1:
      TV.print(15, 40, "SUN");
      break;
    case 2:
      TV.print(15, 40, "MON");
      break;
    case 3:
      TV.print(15, 40, "TUE");
      break;
    case 4:
      TV.print(15, 40, "WED");
      break;
    case 5:
      TV.print(15, 40, "THU");
      break;
    case 6:
      TV.print(15, 40, "FRI");
      break;
    case 7:
      TV.print(15, 40, "SAT");
      break;
  }

  //month
  switch (month) {
    case 1:
      TV.print(48, 40, "JAN");
      break;
    case 2:
      TV.print(48, 40, "FEB");
      break;
    case 3:
      TV.print(48, 40, "MAR");
      break;
    case 4:
      TV.print(48, 40, "APR");
      break;
    case 5:
      TV.print(48, 40, "MAY");
      break;
    case 6:
      TV.print(48, 40, "JUN");
      break;
    case 7:
      TV.print(48, 40, "JUL");
      break;
    case 8:
      TV.print(48, 40, "AUG");
      break;
    case 9:
      TV.print(48, 40, "SEP");
      break;
    case 10:
      TV.print(48, 40, "OCT");
      break;
    case 11:
      TV.print(48, 40, "NOV");
      break;
    case 12:
      TV.print(48, 40, "DEC");
      break;
  }

  //day of month
  //put in code to shif date over a hair when only displaying 1 digit
  if (dayOfMonth < 10) {
    TV.print(81, 40, dayOfMonth);
  }
  else {
    TV.print(78, 40, dayOfMonth);
  }

  //FOURTH LINE
  TV.draw_line(15, 60, 96, 60, 1); //draw a white line 1 is white, 0 is black, 1 is white

  TV.select_font(font4x6);
  TV.print(20, 65, "TEMP:");
  TV.select_font(retro8x16);
  TV.print(51, 65, tempF);
  TV.select_font(font4x6);
  TV.print(71, 66, "F");

  //clean up display...
  //refresh the screen when the hour changes
  if ((minutes == 0) && (seconds == 0)) {

    //this will be called a dozen times in that 1 second that this function
    //is true if I don't put a timer in here to limit how many times it's called

    unsigned long currentMillis = millis();

    if (currentMillis - previousMillis > interval) { //only true for 100ms based on 900ms interval
      previousMillis = currentMillis;

      TV.clear_screen();
    }
  }

  //handle buttons
  //hours button pressed
  if (digitalRead(hourButton) == LOW) {

    hourButtonPressed();
  }

  //minutes button pressed
  if (digitalRead(minuteButton) == LOW) {

    minutesButtonPressed();
  }

  //day button pressed
  if (digitalRead(dayButton) == LOW) {

    dayButtonPressed();
  }

  //month button pressed
  if (digitalRead(monthButton) == LOW) {

    monthButtonPressed();
  }

  //date button pressed
  if (digitalRead(dateButton) == LOW) {

    dateButtonPressed();
  }

  //mode button pressed
  if (digitalRead(modeButton) == LOW) {

    modeButtonPressed();
  }
} //end loop

/////////////////////////////////////////////////////////////////////////////////////////////
//handle buttons
//hours button pressed
void hourButtonPressed() {

  //debounce
  TV.delay(100);
  hours++;

  if ((hours == 1) || (hours == 13)) {
    TV.clear_screen(); //going from 2 digits to 1 leaves garbage on the display, this clears it
  }

  if (hours > 23) {
    hours = 0;
  }

  set_time();
}

//minutes button pressed
void minutesButtonPressed() {

  //debounce
  TV.delay(100);

  minutes++;
  seconds = 0;

  if (minutes > 59) {
    minutes = 0;
  }

  set_time();
}

//day button pressed
void dayButtonPressed() {

  //debounce
  TV.delay(100);

  dayOfWeek++;

  if (dayOfWeek == 8) { //DOW starts at at 1 (SUN), goes to 7 (SAT)
    dayOfWeek = 1; //return to SUN
  }

  set_date();
}

//month button pressed
void monthButtonPressed() {

  //debounce
  TV.delay(100);

  month++;

  if (month == 13) {
    month = 1;
  }

  set_date();
}

//date button pressed
void dateButtonPressed() {

  //debounce
  TV.delay(100);

  dayOfMonth++;

  if (dayOfMonth == 32) { //nevermind 30 vs 31 vs 28, just do worst case
    dayOfMonth = 1;
    TV.clear_screen(); //going from 2 digits to 1 leaves garbage on the display, this clears it
  }

  set_date();
}

//mode button pressed
void modeButtonPressed() {

  //debounce
  TV.delay(100);
  TV.clear_screen();
}

//DS3231
void set_date() {

  Wire.beginTransmission(104);
  Wire.write(3);//set register to 3 (day)
  Wire.write(decToBcd(dayOfWeek));
  Wire.write(decToBcd(dayOfMonth));
  Wire.write(decToBcd(month));
  Wire.write(decToBcd(year));
  Wire.endTransmission();
}

void get_date() {

  Wire.beginTransmission(104);
  Wire.write(3);//set register to 3 (day)
  Wire.endTransmission();
  Wire.requestFrom(104, 4); //get 5 bytes(day,date,month,year,control);
  dayOfWeek   = bcdToDec(Wire.read());
  dayOfMonth  = bcdToDec(Wire.read());
  month = bcdToDec(Wire.read());
  year  = bcdToDec(Wire.read());
}

void set_time() {

  Wire.beginTransmission(104);
  Wire.write(0);
  Wire.write(decToBcd(seconds));
  Wire.write(decToBcd(minutes));
  Wire.write(decToBcd(hours));
  Wire.endTransmission();
}

void get_time() {

  Wire.beginTransmission(104);
  Wire.write(0);//set register to 0
  Wire.endTransmission();
  Wire.requestFrom(104, 3);//get 3 bytes (seconds,minutes,hours);
  seconds = bcdToDec(Wire.read() & 0x7f);
  minutes = bcdToDec(Wire.read());
  hours = bcdToDec(Wire.read() & 0x3f);
}

//SPI stuff for the DS3231 interface
///////////////////////////////////////////////////////////////////////
byte decToBcd(byte val) {
  return ( (val / 10 * 16) + (val % 10) );
}

byte bcdToDec(byte val) {
  return ( (val / 16 * 10) + (val % 16) );
}
